﻿using System;
using System.Configuration;
using System.Configuration.Provider;
using System.Diagnostics;
using System.Web.Configuration;

namespace ACleanCoder.ProviderPattern.Example
{
    internal sealed class Engine
    {
        private static ProviderCollection providers;

        private static EngineProvider currentProvider;

        static Engine()
        {
            try
            {
                Initialise();
            }
            catch (ConfigurationErrorsException ex)
            {
                // logging statements in here
                throw;
            }
        }

        private static void Initialise()
        {
            var engineConfigurationSection =
              ConfigurationManager.GetSection("engineConfiguration") as EngineConfigurationSection;

            if (engineConfigurationSection == null)
            {
                throw new ConfigurationErrorsException("'engineConfiguration' configuration section was not found.");
            }

            providers = new ProviderCollection();

            try
            {
                // populate the providers from the configuration section.
                ProvidersHelper.InstantiateProviders(
                  engineConfigurationSection.Providers, providers, typeof(EngineProvider));
            }
            catch (Exception ex)
            {
                throw new ConfigurationErrorsException("Could not parse 'engineConfiguration' section.", ex);
            }

            // set the providers as read-only so that no-one can tamper with them.
            providers.SetReadOnly();

            if (string.IsNullOrEmpty(engineConfigurationSection.DefaultProvider))
            {
                throw new ConfigurationErrorsException(
                  "No default provider specified for 'engineConfiguration' configuration section.");
            }

            try
            {
                currentProvider = (EngineProvider)providers[engineConfigurationSection.DefaultProvider];
            }
            catch (Exception ex)
            {
                throw new ConfigurationErrorsException(
                "Default 'engineConfiguration' provider not found, or is invalid.", ex);
            }
        }

        public static void Start()
        {
            currentProvider.Start();
        }
    }
}